module orac_mpi
  use rem
  implicit none
  include 'mpif.h'
  !  integer            :: iproc ! iproc is the process rank
  !  integer            :: nproc ! nproc is the total number of processes
  
contains 
  
  !-----------------------------------------------------------
  
  subroutine mpi_initialize(iproc,nproc)
    !   initialize the MPI environment
    implicit none
    integer, intent(out) :: iproc
    integer, intent(out) :: nproc
    ! local variables
    integer :: ierr
    
    CALL MPI_Init(ierr)
    
    CALL MPI_Comm_size(MPI_COMM_WORLD, nproc, ierr)
    
    CALL MPI_Comm_rank(MPI_COMM_WORLD, iproc, ierr)

    ! Before doing anything check whether nproc is less than maxrep
    if(nproc.gt.maxrep) THEN 
       if(iproc.eq.0) THEN
          WRITE(*,101) nproc, maxrep 
101       FORMAT( '************************************************' / &
               '*          FATAL ERROR!!!                      *' / &
               '*  maxrep is less than nproc                   *' / &
               '* maxstep =',i4,'  maxrep =',i4,'              *' / &
               '* Action: Increase maxrep in file rem.f90      *' / &
               '*         and recompile                        *' / &
               '************************************************' /)
          
          CALL MPI_Finalize(ierr)
          STOP
       end if
    end if
    
  end subroutine mpi_initialize

!-----------------------------------------------------------

  subroutine mpi_mkdirs(iproc)
    ! make a different dir for each process 
    ! and put the current wdir in it 
    implicit none
    integer,intent(in) :: iproc
    ! local variables
    integer            :: i
    integer            :: ierr, chdir, getcwd, scan, inull
    character(len=128) :: directory,file,wdir,file2,array,inputfile
    character(len=512) :: command,pwd
    CHARACTER(len=130) :: line
    LOGICAL            :: EXIST

    file="PAR"
    
    !   copy the input file to a generic file INPUT
    IF(iproc .EQ. 0) THEN
       OPEN (unit=99,file="INPUT")
       rw_loop: do 
          READ(5,'(a130)',iostat=ierr) line
          if(ierr < 0) exit rw_loop
          WRITE(99,'(a130)') line
       end do rw_loop
       CLOSE(99)
    END IF
    
    CALL MPI_Barrier(MPI_COMM_WORLD,ierr) 
    
    ! initialize logical variables
    para_restart = .false.

    ! place the current working directory name in wdir
    ! i = 0 on success
    wdir=''
    i=getcwd(wdir)

    ! directory='cwd/PARXXXX'

    ! if wdir ends with a NULL, remove it; 
    ! then, trim trailing blanks
    inull=SCAN(wdir, char(0), .true.) 
    if(inull.ne.0) wdir=wdir(1:inull-1)
    WRITE (directory,'(a,a,a,i4.4)') trim(wdir),'/',trim(file),iproc
      
    exist=.false.

    ! check if the INPUT file is present in the PARXXXX directory
    inputfile = TRIM(directory) // '/INPUT'
    INQUIRE(FILE=inputfile,EXIST=exist) 
    
    ! try to change the work directory to PARXXXX
    CALL FORT_CSTRING(directory,array)
    i=CHDIR(array)

    ! consistency check: 
    ! if there is a PARXXXX directory (i = 0), it must contain 
    ! an INPUT file from some old run
    if(i.eq.0.neqv.exist) THEN 
       write(6,*) ' I/O Problems with process',iproc,i,exist,inputfile
       STOP
    END IF
    
    ! ****************************************************************************************
    ! ************** if a directory PARXXXX exists, cp the new INPUT file in it **************
    ! ****************************************************************************************
    if (i.eq.0) then 
       
       ! the work directory is now PARXXXX
       !     copy the new INPUT file 
       
       command = 'bash -c ' // '"' // 'cp -f ../INPUT . >& /dev/null' // '"'
       CALL system (command)
       
    else
       
       ! ***********************************************************************************
       ! ****************** if there is not a directory PARXXXX, create it  ***************
       ! ***********************************************************************************
       
       ! create directory PARXXXX
       command = 'bash -c ' // '"' // 'mkdir' // ' ' // directory // ' >& /dev/null ' // '"'
       CALL system (command)
       
       ! and copy INPUT file in it 
       command = 'bash -c ' // '"' // 'cp '//'INPUT '//directory // ' >& /dev/null'  // '"'
       
       CALL system (command)
       
       ! change the current work directory 
       i=CHDIR(array)
       
       if (i.ne.0) then
          ! error: stop
          WRITE(*,*) "************************************************"
          WRITE(*,*) "*          FATAL ERROR!!!                      *"
          WRITE(*,*) "*  Fatal error in changing directory           *"
          WRITE(*,*) "*  Check that the subdirectories PARXXXX       *"
          WRITE(*,*) "*  were created.                               *"
          WRITE(*,*) "************************************************"
          CALL MPI_Finalize(ierr)
          STOP 
       endif
    endif
      
    CALL MPI_Barrier(MPI_COMM_WORLD,ierr) 
    
    ! refresh wdir
    
    i=getcwd(wdir)      

  end subroutine mpi_mkdirs

!-----------------------------------------------------------

  subroutine mpi_headings(iproc,nproc)
    implicit none
    integer, intent(in) :: iproc
    integer, intent(in) :: nproc

    if(iproc.eq.0) THEN
       write(*,123) nproc
123    format &
            (10x,'====================================================' &
            /10x,'=                                                  =' &
            /10x,'=    P A R A L L E L    O R A C     S T A R T E D  =' &
            /10x,'=                                                  =' &
            /10x,'=        RUN LAUNCHED ON ',i3,' PROCESSORS            =' &
            /10x,'=                                                  =' &
            /10x,'====================================================')
    end if

  end subroutine mpi_headings

!-----------------------------------------------------------

  subroutine mpi_close(iproc)
    implicit none
    integer, intent(in) :: iproc
    ! local variables 
    integer :: ierr
    
    if(iproc == 0) then 
       WRITE(6,100)
       WRITE(6,200)
    end if
    call MPI_FINALIZE(ierr)
    
100 FORMAT(// &
         '     !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'/ &
         '     !                                                                      !'/ &
         '     !                        PROGRAM COMPLETED                             !'/ &
         '     !                                                                      !'/ &
         '     !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'/) 
    
200 FORMAT(//10x, &
         '|--------------------------------------------------------------|',/10x, &
         "|    ``They're unfriendly.  Which is fortunate really;         |",/10x, &
         "|      they'd be difficult to love.''                          |",/10x, &
         "|                              . . . Avon    Blake's Seven     |",/10x, &
         "|                                                              |",/10x, &
         "|--------------------------------------------------------------|"//)
    
  end subroutine mpi_close

!-----------------------------------------------------------
  
end module orac_mpi
